<?php

namespace Drupal\document_ocr\Entity\Form\Processor;

use Drupal\Core\Form\FormStateInterface;

/**
 * Configuration entity edit form.
 */
class ConfigurationForm extends BaseForm {

  /**
   * {@inheritdoc}
   */
  protected $success_message = '%label processor configuration updated.';

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'document_ocr_processor_' . $this->entity->getProcessor();
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildForm($form, $form_state);
    $form['#attached']['library'][] = 'document_ocr/document_ocr.admin';
    $document_ocr_processor = $this->entity->getProcessorPlugin();
    $form_state->set('document_ocr_processor', $document_ocr_processor);
    $configuration = $this->entity->getConfiguration();
    $document_ocr_processor
      ->setCredentials($this->entity->getCredentials())
      ->setConfiguration($configuration);
    if ($document_ocr_processor->isAsynchronous()) {
      $form['asynchronous'] = [
        '#type' => 'checkbox',
        '#title' => $this->t('Asynchronous requests (best for large files, <em>queue processing only</em>)'),
        '#default_value' => !empty($configuration['asynchronous']),
        '#weight' => 100,
      ];
      $attempt_values = range(3, 20);
      $form['asynchronous_attempts'] = [
        '#type' => 'select',
        '#title' => $this->t('Attempt limit'),
        '#options' => array_combine($attempt_values, $attempt_values),
        '#description' => $this->t('Set the limit for the asynchronous processing attempts. The status of the processing task will be set to <em>Failed</em> upon reaching the limit.'),
        '#default_value' => !empty($configuration['asynchronous_attempts']) ? $configuration['asynchronous_attempts'] : 3,
        '#weight' => 101,
        '#states' => [
          'visible' => [
            ':input[name="asynchronous"]' => ['checked' => TRUE],
          ],
        ],
      ];
      $form['asynchronous_initiate'] = [
        '#type' => 'checkbox',
        '#title' => $this->t('Initiate processing right when mapped entity is saved'),
        '#description' => $this->t('This will allow perform initial async steps like uploading files to the cloud right when mapped entity is saved. This could be noticeably slower page load when saving mapped entities, but will process documents faster.'),
        '#default_value' => !empty($configuration['asynchronous_initiate']),
        '#weight' => 102,
        '#states' => [
          'visible' => [
            ':input[name="asynchronous"]' => ['checked' => TRUE],
          ],
        ],
      ];
    }
    $configuration_form = $document_ocr_processor->configurationForm($form, $form_state);
    if ($configuration_form !== $form) {
      $form = array_merge($form, $configuration_form);
      $form['actions']['submit']['#value'] = $this->t('Update %name', ['%name' => $this->entity->label()]);
    }
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function buildEntity(array $form, FormStateInterface $form_state) {
    /** @var \Drupal\document_ocr\Processor $entity */
    $entity = parent::buildEntity($form, $form_state);
    $document_ocr_processor = $form_state->get('document_ocr_processor');
    if ($configuration = $document_ocr_processor->configurationValues($form, $form_state)) {
      if ($document_ocr_processor->isAsynchronous()) {
        $configuration['asynchronous'] = (bool) $form_state->getValue('asynchronous');
        $configuration['asynchronous_attempts'] = (int) $form_state->getValue('asynchronous_attempts');
        $configuration['asynchronous_initiate'] = (bool) $form_state->getValue('asynchronous_initiate');
      }
      $entity->setConfiguration($configuration);
    }
    return $entity;
  }

}
