<?php

namespace Drupal\document_ocr\ParamConverter;

use Drupal\Core\ParamConverter\ParamConverterInterface;
use Drupal\Component\Plugin\Exception\PluginNotFoundException;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;
use Symfony\Component\Routing\Route;
use Drupal\document_ocr\Plugin\ProcessorManager;

/**
 * Resolves "document_ocr_processor" type parameters in routes.
 */
final class ProcessorConverter implements ParamConverterInterface {

  /**
   * Processor manager.
   *
   * @var \Drupal\document_ocr\Plugin\ProcessorManager
   */
  private $processor;

  /**
   * ProcessorConverter constructor.
   *
   * @param \Drupal\document_ocr\Plugin\ProcessorManager $processor
   *   The Processor plugin manager.
   */
  public function __construct(ProcessorManager $processor) {
    $this->processor = $processor;
  }

  /**
   * {@inheritdoc}
   */
  public function convert($value, $definition, $name, array $defaults) {
    $not_found = new NotFoundHttpException();
    try {
      $processor = $this->processor->createInstance(str_replace('-', '_', $value));
      if ($processor->requirementsAreMet()) {
        return $processor;
      }
      else {
        throw $not_found;
      }
    }
    catch (PluginNotFoundException $e) {
      throw $not_found;
    }
    return NULL;
  }

  /**
   * {@inheritdoc}
   */
  public function applies($definition, $name, Route $route) {
    return (!empty($definition['type']) && $definition['type'] == 'document_ocr_processor');
  }

}
