<?php

namespace Drupal\document_ocr\ParamConverter;

use Drupal\Core\ParamConverter\ParamConverterInterface;
use Drupal\Component\Plugin\Exception\PluginNotFoundException;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;
use Symfony\Component\Routing\Route;
use Drupal\document_ocr\Plugin\TransformerManager;

/**
 * Resolves "document_ocr_transformer" type parameters in routes.
 */
final class TransformerConverter implements ParamConverterInterface {

  /**
   * Transformer manager.
   *
   * @var \Drupal\document_ocr\Plugin\TransformerManager
   */
  private $transformer;

  /**
   * TransformerConverter constructor.
   *
   * @param \Drupal\document_ocr\Plugin\TransformerManager $transformer
   *   The Transformer plugin manager.
   */
  public function __construct(TransformerManager $transformer) {
    $this->transformer = $transformer;
  }

  /**
   * {@inheritdoc}
   */
  public function convert($value, $definition, $name, array $defaults) {
    $not_found = new NotFoundHttpException();
    try {
      $transformer = $this->transformer->createInstance(str_replace('-', '_', $value));
      if ($transformer->requirementsAreMet()) {
        return $transformer;
      }
      else {
        throw $not_found;
      }
    }
    catch (PluginNotFoundException $e) {
      throw $not_found;
    }
    return NULL;
  }

  /**
   * {@inheritdoc}
   */
  public function applies($definition, $name, Route $route) {
    return (!empty($definition['type']) && $definition['type'] == 'document_ocr_transformer');
  }

}
